@@ -2,13 +2,31 @@
 # :license: MIT, see LICENSE for more details.
 
 """
-yaspin.yaspin
-~~~~~~~~~~~~~
+yaspin.core
+~~~~~~~~~~~
 
 A lightweight terminal spinner.
 """
+
 from __future__ import annotations
 
+from collections.abc import Generator, Iterator, Sequence
+from contextlib import contextmanager
+from dataclasses import dataclass
+from datetime import timedelta
+from typing import (
+    Any,
+    Callable,
+    cast,
+    Final,
+    Optional,
+    Protocol,
+    runtime_checkable,
+    TYPE_CHECKING,
+    TypeVar,
+    Union,
+)
+
 import functools
 import itertools
 import shutil
@@ -17,27 +35,8 @@ import sys
 import threading
 import time
 import warnings
-from contextlib import contextmanager
-from dataclasses import dataclass
-from datetime import timedelta
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Final,
-    Generator,
-    Iterator,
-    Optional,
-    Protocol,
-    Sequence,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-    runtime_checkable,
-)
 
-from termcolor import ATTRIBUTES, COLORS, HIGHLIGHTS, colored
+from termcolor import ATTRIBUTES, colored, COLORS, HIGHLIGHTS
 
 from .constants import SPINNER_ATTRS
 
@@ -51,7 +50,7 @@ Fn = TypeVar("Fn", bound=Callable[..., Any])
 ENCODING: Final[str] = "utf-8"
 
 
-def to_unicode(text_type: Union[str, bytes], encoding: str = ENCODING) -> str:
+def to_unicode(text_type: str | bytes, encoding: str = ENCODING) -> str:
     if isinstance(text_type, bytes):
         return text_type.decode(encoding)
     return text_type
@@ -71,7 +70,7 @@ class SignalHandlerProtocol(Protocol):
     def __call__(self, signum: int, frame: Any, spinner: Yaspin) -> None: ...
 
 
-def default_handler(signum: int, frame: Any, spinner: Yaspin) -> None:  # pylint: disable=unused-argument
+def default_handler(signum: int, frame: Any, spinner: Yaspin) -> None:
     """Signal handler, used to gracefully shut down the ``spinner`` instance
     when specified signal is received by the process running the ``spinner``.
 
@@ -83,7 +82,7 @@ def default_handler(signum: int, frame: Any, spinner: Yaspin) -> None:  # pylint
     sys.exit(0)
 
 
-def fancy_handler(signum: int, frame: Any, spinner: Yaspin) -> None:  # pylint: disable=unused-argument
+def fancy_handler(signum: int, frame: Any, spinner: Yaspin) -> None:
     """Signal handler, used to gracefully shut down the ``spinner`` instance
     when specified signal is received by the process running the ``spinner``.
 
@@ -95,7 +94,7 @@ def fancy_handler(signum: int, frame: Any, spinner: Yaspin) -> None:  # pylint:
     sys.exit(0)
 
 
-class Yaspin:  # pylint: disable=too-many-instance-attributes
+class Yaspin:
     """Implements a context manager that spawns a thread
     to write spinner frames into a tty (stdout) during
     context execution.
@@ -105,16 +104,16 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
     # it sets the sys.stdout.encoding attribute to the terminal's encoding.
     # The print statement's handler will automatically encode unicode
     # arguments into bytes.
-    def __init__(  # pylint: disable=too-many-arguments
+    def __init__(
         self,
         spinner: Spinner = default_spinner,
         text: str = "",
-        color: Optional[str] = None,
-        on_color: Optional[str] = None,
-        attrs: Optional[Sequence[str]] = None,
+        color: str | None = None,
+        on_color: str | None = None,
+        attrs: Sequence[str] | None = None,
         reversal: bool = False,
         side: str = "left",
-        sigmap: Optional[dict[signal.Signals, SignalHandlers]] = None,
+        sigmap: dict[signal.Signals, SignalHandlers] | None = None,
         timer: bool = False,
         ellipsis: str = "",
     ) -> None:
@@ -137,14 +136,14 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         self._timer = timer
         self._ellipsis = ellipsis
         self._terminal_width: int = shutil.get_terminal_size().columns
-        self._start_time: Optional[float] = None
-        self._stop_time: Optional[float] = None
+        self._start_time: float | None = None
+        self._stop_time: float | None = None
 
         # Helper flags
-        self._stop_spin: Optional[threading.Event] = None
-        self._hide_spin: Optional[threading.Event] = None
-        self._spin_thread: Optional[threading.Thread] = None
-        self._last_frame: Optional[str] = None
+        self._stop_spin: threading.Event | None = None
+        self._hide_spin: threading.Event | None = None
+        self._spin_thread: threading.Thread | None = None
+        self._last_frame: str | None = None
         self._stdout_lock = threading.Lock()
         self._hidden_level = 0
         self._cur_line_len = 0
@@ -166,9 +165,9 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         if self._spin_thread is None:
             raise RuntimeError("spin thread is None")
@@ -187,7 +186,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
     def __getattr__(self, name: str) -> Yaspin:
         # CLI spinners
         if name in SPINNER_ATTRS:
-            from .spinners import Spinners  # pylint: disable=import-outside-toplevel
+            from .spinners import Spinners
 
             sp = getattr(Spinners, name)
             self.spinner = sp
@@ -198,9 +197,9 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             if name in ATTRIBUTES:
                 self.attrs = [name]  # calls property setter
             if name in COLORS:
-                setattr(self, "color", name)  # calls property setter
+                self.color = name  # calls property setter
             if name in HIGHLIGHTS:
-                setattr(self, "on_color", name)  # calls property setter
+                self.on_color = name  # calls property setter
         # Side: "left" or "right"
         elif name in ("left", "right"):
             self.side = name  # calls property setter
@@ -231,7 +230,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         self._text = txt
 
     @property
-    def color(self) -> Optional[str]:
+    def color(self) -> str | None:
         return self._color
 
     @color.setter
@@ -240,7 +239,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         self._color_func = self._compose_color_func()  # update
 
     @property
-    def on_color(self) -> Optional[str]:
+    def on_color(self) -> str | None:
         return self._on_color
 
     @on_color.setter
@@ -295,6 +294,18 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
     # Public
     #
     def start(self) -> None:
+        """
+        Start the spinner animation in a separate thread.
+
+        Initializes and starts the spinner animation by hiding the cursor, recording
+        the start time, and creating a new thread to run the spinner. It also
+        sets up necessary threading events to control the spinner's behavior.
+
+        If signal handlers are registered, they will be set up before starting the spinner.
+
+        In case of any failure that prevents the spinner from starting, the cursor will
+        be shown to ensure it is not left hidden.
+        """
         if self._sigmap:
             self._register_signal_handlers()
 
@@ -313,6 +324,16 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             self._show_cursor()
 
     def stop(self) -> None:
+        """
+        Stops the spinner and performs necessary cleanup.
+
+        Records the stop time, resets signal handlers to their default if
+        they were modified, stops the spinning thread, clears the spinner
+        line, and shows the cursor.
+
+        Raises:
+            RuntimeError: If the stop_spin event is None.
+        """
         self._stop_time = time.time()
 
         if self._dfl_sigmap:
@@ -329,7 +350,17 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         self._show_cursor()
 
     def hide(self) -> None:
-        """Hide the spinner to allow for custom writing to the terminal."""
+        """
+        Hide the spinner to allow for custom writing to the terminal.
+
+        Sets a flag to indicate that the spinner should be hidden, clears
+        the current line in the terminal, and flushes the stdout buffer.
+        It ensures that the spinner thread is alive and the hide flag is
+        not already set before performing these actions.
+
+        Raises:
+            RuntimeError: If the hide_spin attribute is None.
+        """
         thr_is_alive = self._spin_thread and self._spin_thread.is_alive()
         if self._hide_spin is None:
             raise RuntimeError("hide_spin is None")
@@ -346,7 +377,15 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
 
     @contextmanager
     def hidden(self) -> Generator[None, None, None]:
-        """Hide the spinner within a block, can be nested"""
+        """
+        Temporarily hides the spinner within a context block. This method can be nested.
+
+        When the context is entered, the spinner is hidden if it is not already hidden.
+        When the context is exited, the spinner is shown again if it was hidden by this method.
+
+        Yields:
+            None: This method is a generator that yields control back to the caller.
+        """
         if self._hidden_level == 0:
             self.hide()
         self._hidden_level += 1
@@ -358,7 +397,17 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
                 self.show()
 
     def show(self) -> None:
-        """Show the hidden spinner."""
+        """
+        Show the hidden spinner.
+
+        Checks if the spinner thread is alive and if the spinner is currently
+        hidden. If both conditions are met, it clears the hidden spinner
+        flag and clears the current line to ensure the spinner is not
+        appended to it.
+
+        Raises:
+            RuntimeError: If the `_hide_spin` attribute is `None`.
+        """
         thr_is_alive = self._spin_thread and self._spin_thread.is_alive()
         if self._hide_spin is None:
             raise RuntimeError("hide_spin is None")
@@ -371,15 +420,20 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
                 self._clear_line()
 
     def write(self, text: str) -> None:
-        """Write text in the terminal without breaking the spinner."""
+        """
+        Write text in the terminal without breaking the spinner.
+
+        Ensures that the spinner is temporarily cleared, the text
+        is written to the terminal, and then the spinner is restored.
+
+        Args:
+            text (str): The text to be written to the terminal.
+        """
         # similar to tqdm.write()
         # https://pypi.python.org/pypi/tqdm#writing-messages
         with self._stdout_lock:
             self._clear_line()
-            if isinstance(text, (str, bytes)):
-                _text = to_unicode(text)
-            else:
-                _text = str(text)
+            _text = to_unicode(text) if isinstance(text, (str, bytes)) else str(text)
             sys.stdout.write(f"{_text}\n")
             self._cur_line_len = 0
 
@@ -403,7 +457,18 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         )
 
     def _freeze(self, final_text: str) -> None:
-        """Stop spinner, compose last frame and 'freeze' it."""
+        """
+        Stop the spinner and display the final frame.
+
+        Stops the spinner, composes the last frame with the provided final text,
+        and 'freezes' it by writing the final frame to the standard output.
+
+        Args:
+            final_text (str): The final text to be displayed when the spinner stops.
+
+        Raises:
+            RuntimeError: If the last frame is None.
+        """
         text = to_unicode(final_text)
         self._last_frame = self._compose_out(text, mode="last")
 
@@ -417,6 +482,16 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             self._cur_line_len = 0
 
     def _spin(self) -> None:
+        """
+        Handles the spinning animation.
+
+        Continuously updates the spinner's output on the terminal until
+        the `_stop_spin` event is set. If the `_hide_spin` event is set,
+        it temporarily pauses the spinning.
+
+        Raises:
+            RuntimeError: If `_stop_spin` is None.
+        """
         if self._stop_spin is None:
             raise RuntimeError("stop_spin is None")
 
@@ -440,7 +515,14 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             # Wait
             self._stop_spin.wait(self._interval)
 
-    def _compose_color_func(self) -> Optional[Callable[..., str]]:
+    def _compose_color_func(self) -> Callable[..., str] | None:
+        """
+        Compose a color function based on the current environment.
+
+        If the environment is Jupyter, returns None as ANSI color control sequences
+        are problematic in Jupyter notebooks. Otherwise, returns a partial function
+        that applies the specified color, background color, and attributes to text.
+        """
         if self.is_jupyter():
             # ANSI Color Control Sequences are problematic in Jupyter
             return None
@@ -452,15 +534,31 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             attrs=list(self._attrs),
         )
 
-    def _compose_out(self, frame: str, mode: Optional[str] = None) -> str:
+    def _compose_out(self, frame: str, mode: str | None = None) -> str:
+        """
+        Compose the output string for the spinner.
+
+        Args:
+            frame (str): The current frame of the spinner animation.
+            mode (str, optional): The mode in which the output is generated. If None,
+                                  the output is generated on the same line with a carriage
+                                  return. If a value is provided, the output is generated
+                                  on a new line.
+
+        Returns:
+            str: The composed output string including the spinner frame, text, timer,
+                 and any specified colors and positions.
+
+        Raises:
+            ValueError: If the terminal size is too small to display the spinner with
+                        the given settings.
+        """
         text = str(self._text)
 
         # Timer
         if self._timer:
             sec, fsec = divmod(round(100 * self.elapsed_time), 100)
-            timer = " ({}.{:02.0f})".format(  # pylint: disable=consider-using-f-string
-                timedelta(seconds=sec), fsec
-            )
+            timer = f" ({timedelta(seconds=sec)}.{fsec:02.0f})"
         else:
             timer = ""
 
@@ -468,7 +566,8 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         max_text_len = self._get_max_text_length(len(frame), len(timer))
         if max_text_len < 1:
             raise ValueError(
-                f"Terminal size {self._terminal_width} is too small to display spinner with the given settings."
+                f"Terminal size {self._terminal_width} is too small to display spinner "
+                "with the given settings."
             )
         text = text[:max_text_len] + self._ellipsis if len(text) > max_text_len else text
 
@@ -481,14 +580,21 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             frame, text = text, frame
 
         # Mode
-        if mode is None:
-            out = f"\r{frame} {text}{timer}"
-        else:
-            out = f"{frame} {text}{timer}\n"
+        out = f"\r{frame} {text}{timer}" if mode is None else f"{frame} {text}{timer}\n"
 
         return out
 
     def _get_max_text_length(self, frame_width: int, timer_width: int) -> int:
+        """
+        Calculate the maximum length of text that can be displayed within the terminal width.
+
+        Args:
+            frame_width (int): The width of the frame.
+            timer_width (int): The width of the timer.
+
+        Returns:
+            int: The maximum length of text that can be displayed.
+        """
         ellipsis_width = len(self._ellipsis)
         # There is always a space between frame and text
         frame_width += 1
@@ -496,6 +602,17 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         return self._terminal_width - frame_width - timer_width - ellipsis_width
 
     def _register_signal_handlers(self) -> None:
+        """
+        Registers custom signal handlers for the spinner.
+
+        Sets up signal handlers defined in the `_sigmap` attribute.
+        It ensures that SIGKILL is not included. For each signal in
+        `_sigmap`, stores the default signal handler for later restoration
+        during the cleanup phase.
+
+        Raises:
+            ValueError: If an attempt is made to set a handler for the SIGKILL signal.
+        """
         # SIGKILL cannot be caught or ignored, and the receiving
         # process cannot perform any clean-up upon receiving this
         # signal.
@@ -524,6 +641,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
             signal.signal(sig, sig_handler)
 
     def _reset_signal_handlers(self) -> None:
+        """Resets the signal handlers to their default values."""
         for sig, sig_handler in self._dfl_sigmap.items():
             signal.signal(sig, sig_handler)
 
@@ -540,9 +658,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
 
         if value not in COLORS:
             raise ValueError(
-                "'{0}': unsupported color value. Use one of the: {1}".format(  # pylint: disable=consider-using-f-string
-                    value, ", ".join(COLORS.keys())
-                )
+                "'{}': unsupported color value. Use one of the: {}".format(value, ", ".join(COLORS.keys()))
             )
         return value
 
@@ -553,8 +669,9 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
 
         if value not in HIGHLIGHTS:
             raise ValueError(
-                "'{0}': unsupported on_color value. "  # pylint: disable=consider-using-f-string
-                "Use one of the: {1}".format(value, ", ".join(HIGHLIGHTS.keys()))
+                "'{}': unsupported on_color value. Use one of the: {}".format(
+                    value, ", ".join(HIGHLIGHTS.keys())
+                )
             )
         return value
 
@@ -566,18 +683,16 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         for attr in attrs:
             if attr not in ATTRIBUTES:
                 raise ValueError(
-                    "'{0}': unsupported attribute value. "  # pylint: disable=consider-using-f-string
-                    "Use one of the: {1}".format(attr, ", ".join(ATTRIBUTES.keys()))
+                    "'{}': unsupported attribute value. Use one of the: {}".format(
+                        attr, ", ".join(ATTRIBUTES.keys())
+                    )
                 )
         return set(attrs)
 
     @staticmethod
     def _set_spinner(spinner: Spinner) -> Spinner:
         if hasattr(spinner, "frames") and hasattr(spinner, "interval"):
-            if not spinner.frames or not spinner.interval:
-                sp = default_spinner
-            else:
-                sp = spinner
+            sp = default_spinner if not spinner.frames or not spinner.interval else spinner
         else:
             sp = default_spinner
 
@@ -590,7 +705,21 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         return side
 
     @staticmethod
-    def _set_frames(spinner: Spinner, reversal: bool) -> Union[str, Sequence[str]]:
+    def _set_frames(spinner: Spinner, reversal: bool) -> str | Sequence[str]:
+        """
+        Set the frames for the spinner, optionally reversing them.
+
+        Args:
+            spinner (Spinner): The spinner object containing the frames.
+            reversal (bool): If True, the frames will be reversed.
+
+        Returns:
+            Union[str, Sequence[str]]: The frames to be used for the spinner.
+            This can be a single string of frames or a sequence of frame strings.
+
+        Raises:
+            ValueError: If no frames are found in the spinner.
+        """
         uframes = None  # unicode frames
         uframes_seq = None  # sequence of unicode frames
 
@@ -626,7 +755,7 @@ class Yaspin:  # pylint: disable=too-many-instance-attributes
         return spinner.interval * 0.001
 
     @staticmethod
-    def _set_cycle(frames: Union[str, Sequence[str]]) -> Iterator[str]:
+    def _set_cycle(frames: str | Sequence[str]) -> Iterator[str]:
         return itertools.cycle(frames)
 
     @staticmethod
